////////////////////////////////////////////////////////////////////////////////
// TWizard                                                                    //
////////////////////////////////////////////////////////////////////////////////
// Visible graphic component for easy making wizardforms etc.                 //
// Based on TgraphicControl                                                   //
////////////////////////////////////////////////////////////////////////////////
// Version 1.00 Beta                                                          //
// Created                    : 25/07/1998                                    //
// Last modified              : 25/07/1998                                    //
////////////////////////////////////////////////////////////////////////////////
// R. Andr Inghilleri                                                        //
// andre@norleas.no                                                           //
////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTICE :                                                         //
//                                                                            //
//                                                                            //
// This program is FreeWare                                                   //
//                                                                            //
// Please do not release modified versions of this source code.               //
// If you've made any changes that you think should have been there,          //
// feel free to submit them to me.                                            //
////////////////////////////////////////////////////////////////////////////////
//  REVISIONS :                                                               //
//                                                                            //
//  1.10  * Added Gradient fill to background                                 //
//        * Added downsizing of bitmaps in small_x3                           //
//                                                                            //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////


unit Wizard;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, comctrls;

const mxcolors=100;
type
  TPicControl = (Large_x1, Small_x3);
  TDirection = (bdFlat, bdDown, bdLeft, bdRight, bdUp);
  TGrArray=array[1..mxcolors] of tcolor;

type
  TWizard = class(TGraphicControl)
  private
    FDir: TDirection;
    FPicture: TPicture;
    FAlignment: TAlignment;
    FPicControl: TPicControl;
    FTransparent: Boolean;
    col_1 :tcolor;
    ga:tgrarray;

    Procedure CalcColGradients(col1,col2:tcolor;n:integer);
    procedure PictureChanged(Sender: TObject);
    procedure SetAlignment(Value: TAlignment);
    procedure SetColor1(Clr: TColor);
    procedure SetDir(Dir: TDirection);
    procedure SetPicture(Value: TPicture);
    procedure SetPicControl(Value: TPicControl);
    procedure SetTransparent(Value: Boolean);
    procedure FillOneWay(WType: TDirection);
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
  protected
    function DestRect(pic_no,pic_tot:integer): TRect;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas;
  published
    property Align;
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property BackGroundEffect: TDirection read FDir write SetDir default bdUp;
    property Caption;
    property ColorFade:TColor read col_1 write SetColor1 default clNavy;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property Picture: TPicture read FPicture write SetPicture;
    property PictureControl: TPicControl read FPicControl write SetPicControl default Small_x3;
    property PopupMenu;
    property ShowHint;
    property Transparent: Boolean read FTransparent write SetTransparent default True;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

{ TWizard }

constructor TWizard.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  Width := 100;
  Height := 100;
  FDir  := bdUp;
  color := clBlue;
  col_1 := clNavy;
  transparent:=true;
  alignment:=taLeftJustify;
  picturecontrol:=Small_x3;
  if csDesigning in ComponentState
    then picture.bitmap.LoadFromResourceName(HInstance, 'Wiz1');    
  inherited Align := alLeft;
end;

destructor TWizard.Destroy;
begin
  FPicture.Free;
  inherited Destroy;
end;

procedure TWizard.CalcColGradients(col1,col2:tcolor;n:integer);
var i:integer;
    ar,ag,ab,br,bb,bg:integer;
    kr,kg,kb:real;
begin
  ar:=GetRValue(col1);br:=GetRValue(col2);
  AG:=GetGValue(col1);bG:=GetGValue(col2);
  AB:=GetBValue(col1);bB:=GetBValue(col2);
  kr:=(br-ar) /n;
  kg:=(bg-ag) /n;
  kb:=(bb-ab) /n;
  for i:=0 to n-1 do
  ga[i+1]:=rgb(trunc(ar + i*kr),trunc(ag+i*kg),trunc(ab+i*kb));
end;


procedure TWizard.PictureChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWizard.SetAlignment(Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;

procedure TWizard.SetColor1(Clr: TColor);
begin
   col_1:=clr;
   Repaint;
end;

procedure TWizard.SetDir(Dir: TDirection);
begin
   FDir := Dir;
   Repaint;
end;

procedure TWizard.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TWizard.SetPicControl(value: TPicControl);
begin
  if FPicControl <> value then
  begin
    FPicControl := value;
    invalidate;
  end;
end;

procedure TWizard.SetTransparent(Value: Boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    invalidate;
  end;
end;


procedure TWizard.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_CENTER, DT_LEFT, DT_RIGHT);
var picture_number,picture_count, text_height, text_top:integer;
    drawstyle:word;
    rectb:trect;
begin
  if PictureControl=Small_x3 then picture_count:=3
                             else picture_count:=1;
  rectb:=Rect(5, 0, width-10, 50);
  Canvas.Font := Font;
  Canvas.Pen.Style := psclear;
  Canvas.Brush.Style := bssolid;
  Canvas.brush.color:=color;
  if FDir = bdFlat
    then canvas.Rectangle(0, 0, Width, Height)
    else FillOneWay(FDir);

  Canvas.Brush.Style := bsclear;
  if Picture.Graphic <> nil then Picture.Graphic.Transparent := ftransparent;
  DrawStyle := Alignments[FAlignment] or DT_WORDBREAK;
  text_height:=DrawText(Canvas.Handle, PChar(caption), -1, Rectb, DT_CALCRECT or drawstyle);
  text_top:=clientheight-10-text_height;
  rectb:=Rect(5, text_top, width-10,text_top+text_height);
  try
    with inherited Canvas do begin
      DrawText(Canvas.Handle, PChar(caption), -1, Rectb, drawstyle);
      for picture_number:=1 to picture_count do
        StretchDraw(DestRect(picture_number,picture_count), Picture.Graphic);
    end;
  finally
   { FDrawing := Save;  }
  end;
end;

function TWizard.DestRect(pic_no,pic_tot:integer): TRect;
var pic_x ,pic_y, pic_gap:integer;
    pic_rect :Trect;
    verdi    :real;
begin
  pic_x:=picture.width;  pic_y:=picture.height;
  if pic_tot=3 then begin
    if pic_x>(width/2.5) then begin
      verdi:=(width/2.5)/pic_x;
      pic_x:=Round(pic_x * verdi);
      pic_y:=Round(pic_y * verdi);
    end;
    pic_gap:=round((width-(2*pic_x))/3);
    case pic_no of
      1: pic_rect:=rect(pic_gap, 25, pic_gap+pic_x, 25+pic_y);
      2: pic_rect:=rect(pic_gap*2+pic_x, 25, pic_gap*2+pic_x*2, 25+pic_y);
      3: pic_rect:=rect(round((width-pic_x)/2), 25+pic_gap+pic_y, round((width-pic_x)/2)+pic_x, 25+pic_gap+pic_y*2);
    end;
    Result := pic_rect;
  end else if pic_tot=1 then begin
    if (pic_x/width)>(pic_y/Height)
      then verdi:=pic_x/(width-10)
      else verdi:=pic_y/height;
    Result := Bounds((Width-round(Pic_x/verdi)) div 2, 10, round(Pic_x/verdi), round(Pic_y/verdi))
  end;
end;

procedure TWizard.FillOneWay(WType: TDirection);
var verdi   :integer;
    surface :trect;
begin
  if wtype in[bdUp,bdleft] then CalcColGradients(color,col_1,mxcolors)
                        else CalcColGradients(col_1,color,mxcolors);
  surface:=rect(0,0,Width, Height);
  for verdi := 0 to mxcolors-1 do
  begin
    if Wtype in [bdUp,bdDown]
      then Surface.Top  := MulDiv (verdi, Height, mxcolors)   // bottom verdi+1
      else Surface.Left := MulDiv (verdi, Width, mxcolors); // rigth verdi+1
    canvas.Brush.Color := ga[verdi+1];
    canvas.FillRect (surface);
  end;
end;


procedure TWizard.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;


// Register the component


procedure Register;
begin
  RegisterComponents('Graphics', [TWizard]);
end;

end.
